<?php
/**
 * Email Notification For Protected Post
 *
 * @package     email_notification_protected_post
 * @author      The Wolfiz Team
 * @copyright   2018 Wolfiz PVT LTD.
 * @license     GPL-2.0+
 *
 * @wordpress-plugin
 * Plugin Name: Email Notification For Protected Post
 * Plugin URI:  https://wolfiz.com
 * Description: Sends email to specified users with password for protected posts of various post types.
 * Version:     1.0.0
 * Author:      The Wolfiz Team
 * Author URI:  https://wolfiz.com/contact-us/
 * License:     GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
 *
 */


if( !class_exists( 'Wolfiz_Email_Notification_Protected_Post' ) ){
	
	class Wolfiz_Email_Notification_Protected_Post{
		
		public function __construct(){
			register_activation_hook( __FILE__, array( &$this, 'wolfiz_activation_email_notification_protected_post' ) );
			
			add_action( 'admin_enqueue_scripts', array( &$this, 'wolfiz_plugin_add_main_scripts' ) );
			
			add_action( 'admin_enqueue_scripts', array( &$this, 'wolfiz_plugin_add_main_styles' ) );
			
			add_action( 'admin_menu', array( &$this, 'wolfiz_add_proposal_post_type_logs_menu' ) );
			
			add_action( 'add_meta_boxes', array( &$this, 'wolfiz_add_protected_custom_meta_box' ) );
			
			add_action( 'wp_ajax_send_email_notification', array( &$this, 'wolfiz_send_email_notification_ajax_handler' ) );
			
			add_action( 'admin_post_save_notification_email_content', array( &$this,'wolfiz_save_notification_email_content' ) );
			
			add_action( 'admin_post_save_post_types', array( &$this, 'wolfiz_save_post_types' ) );
			
			add_filter( 'wp_mail_from', array( &$this,'wolfiz_aelieve_wp_mail_from' ) );
			
			add_filter( 'wp_mail_from_name', array( &$this, 'wolfiz_aelieve_wp_mail_name' ) );
			
			add_filter( 'tiny_mce_before_init', array( &$this,'wolfiz_format_email_content_editor' ) );
			
		}
		
		public function wolfiz_activation_email_notification_protected_post(){
			global $wpdb; 
			
			$charset_collate = $wpdb->get_charset_collate();
			
			$sql = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}mail_logs` (
				`ID` BIGINT(50) UNSIGNED NOT NULL AUTO_INCREMENT,
				`proposal` TEXT NOT NULL,
				`date_sent` DATETIME NOT NULL,
				`time_sent` TIME NOT NULL,
				`clicks` BIGINT(50) UNSIGNED NOT NULL,
				`email_addresses` VARCHAR(50) NOT NULL,
				`proposal_permalink` TEXT NOT NULL,
				PRIMARY KEY(`ID`)
			)$charset_collate;";	
			
			$wpdb->query( $sql ); 
			
			$this->wolfiz_email_notification_setting_default_post_types();
			$this->wolfiz_email_notification_default_email_settings();
			
		}
		
		
		function wolfiz_email_notification_setting_default_post_types(){
			
			$selected_post_types = array();
			$selected_post_types = get_option( 'selected_post_types' );
			
			if( empty( $selected_post_types ) ){
				$selected_post_types[] = 'post';
				$selected_post_types[] = 'page';
			}
			
			update_option( 'selected_post_types', $selected_post_types );
			
		}
		
		function wolfiz_email_notification_default_email_settings(){
			$email_settings_opts = get_option( 'wolfiz_email_settings' ); 
			
			if( empty( $email_settings_opts['email_body'] ) ){
				
				$email_settings_opts['email_body'] = '<p style="text-align: center;"><img src="[{logo_url}]"></p>';
				$email_settings_opts['email_body'] .= '<p>Hello Sir!</p>';
				$email_settings_opts['email_body'] .= '<br />';
				$email_settings_opts['email_body'] .= '<p>You have been assigned to view post with following details:</p>';
				$email_settings_opts['email_body'] .= '<br />';
				$email_settings_opts['email_body'] .= '<p><strong>Post Name:</strong> [{Title}]</p>';
				$email_settings_opts['email_body'] .= '<br />';
				$email_settings_opts['email_body'] .= '<p><strong>URL:</strong> [{Permalink}]</p>';
				$email_settings_opts['email_body'] .= '<br />';
				$email_settings_opts['email_body'] .= '<p><strong>Password:</strong> [{Password}]</p>';
				$email_settings_opts['email_body'] .= '<br />';
				$email_settings_opts['email_body'] .= '<p style = "text-align:right"><strong>Regards</strong></p>';
			}
			
			if( empty( $email_settings_opts['from_email_address'] ) ){
				
				$email_settings_opts['from_email_address'] = get_option( 'admin_email' );
			} 
			
			if( empty( $email_settings_opts['from_email_name'] ) ){
				$email_settings_opts['from_email_name'] = get_option( 'blogname' );
				
			}
			
			update_option( 'wolfiz_email_settings', $email_settings_opts ); 
			
		}
		
		
		
		public function wolfiz_add_protected_custom_meta_box(){
			
			$selected_post_types = array();
			$selected_post_types = get_option( 'selected_post_types' );
			
			add_meta_box( 
				'protect_password_email_notification_mb', 
				'Send Email', 
				array( &$this,'wolfiz_protect_password_email_notification_mb_callback' ), 
				$selected_post_types, 
				'side', 
				'high'
			);
		
		}
		
		public function wolfiz_plugin_add_main_scripts(){
			
			wp_enqueue_script( 'jquery' );
			
			wp_enqueue_script( 
				'main-email-send-js', 
				plugin_dir_url( __FILE__ ).'admin/js/main.js', 
				array( 'jquery' ), 
				1.0, 
				true 
			);
			
			wp_localize_script(
				'main-email-send-js',
				'js_obj',
				array(
					'ajaxUrl' => esc_url( admin_url( 'admin-ajax.php' ) ),
					'send_nonce' => wp_create_nonce( 'my_send_email_meta_box_nonce' )
				)
			
			);
			
		}
		
		
		public function wolfiz_plugin_add_main_styles(){
			wp_enqueue_style( 
				'wolfiz-admin-main-css', 
				plugin_dir_url( __FILE__ ).'admin/css/main.css', 
				array(), 
				1.0
			);
			
		}
		
		public function wolfiz_protect_password_email_notification_mb_callback( $post ){
			?>
			
				<div>
					<label for="add_users_textarea"><?php _e( 'Add Users' ); ?>:</label><br />
					<span><input type = "email" id = "add_email_users" name = "add_email_users" /></span>
					<span><a class = "send_users_email button" id = "<?php echo $post->ID; ?>" href = "#" class = "button"><?php _e( 'Send Email'); ?></a></span>
				</div>
			
			<?php
		}
		
		public function wolfiz_send_email_notification_ajax_handler(){
			
			check_ajax_referer( 'my_send_email_meta_box_nonce', 'security_nonce' );
			
			$users_emails = explode( ',', $_POST['emails'] );
			
			$post_permalink = esc_url( get_the_permalink( absint( $_POST['post_id'] ) ) );
			
			$the_post = get_post( absint( $_POST['post_id'] ) );
			
			if( !post_password_required( absint( $_POST['post_id'] ) ) ){
				wp_send_json_error( 
					
					array(
						'success' => false,
						'error' => 'This post is not password protected. Please change it\'s visibility to password protected and add a password to it then update post.'
					)	
				);
				
			}
			
			foreach( $users_emails as $users_email ){
				
				if( !is_email( $users_email ) ){
					
					wp_send_json_error( 
					
						array(
							'success' => false,
							'error' => 'This email address '.$users_email.' is not invalid. Please enter a valid email address. Check "@" is correctly added.'
						)
						
					);
				
				} else {
					$opts = get_option( 'wolfiz_email_settings' );
					
					$to = sanitize_email( $users_email );
					
					$subject = sanitize_text_field( 'Access Assigned to View Post' );
					
					$body = str_replace('[{logo_url}]', sanitize_url( 'https://storage.googleapis.com/stateless-aelieve-com/aelieve/adcb6546-core-aelieve-logo-square-text-hexartboard-1152-px.png' ), wp_kses_post( $opts['email_body'] ) );
					
					$body = str_replace('[{Title}]', $subject, $body );
					
					$body = str_replace('[{Permalink}]', $post_permalink, $body );
					
					$body = str_replace('[{Email_Permalink}]', $post_permalink.'?user_email='.$to, $body );
				
					$body = str_replace('[{Email}]', $to, $body );
					
					$body = str_replace('[{Password}]', $the_post->post_password, $body );
					
					$headers[] = 'Content-Type: text/html; charset=UTF-8'; 
					
					wp_mail( $to, $subject, $body, $headers );
				}
			}
			$this->wolfiz_add_mail_log_data( $subject, $users_emails, $post_permalink );
			wp_send_json_success( 
				array(
					'success' => true, 
					'email_send' => 'Emails have been sent to respective email addresses.'
				) 
			);
			
		}
		
		public function wolfiz_save_post_types(){
			
			$selected_post_types = $_POST['selected_post_types'];
			
			update_option( 'selected_post_types', $selected_post_types );
			
			wp_redirect( esc_url( admin_url().'admin.php?page=wolfiz_aelieve_post_types&tab=1' ) ); exit;
		} 
		
		
		public function wolfiz_add_mail_log_data( $subject, $users_emails, $post_permalink ){
			global $wpdb;
			
			
			$date_time = new DateTime();
			$current_dt = $date_time->format( 'Y-m-d H:i:s' );
			$current_tm = $date_time->format( 'H:i:s' );
			
			$subject = sanitize_text_field( $subject );
			$post_permalink = sanitize_url( $post_permalink );
			
			$clicks = 0; 
			
			
			foreach( $users_emails as $users_email ){
				
				$users_email = sanitize_email( $users_email );
				$sql = "INSERT INTO `{$wpdb->prefix}mail_logs` ( `proposal`, `date_sent`, `time_sent`, `clicks`, `email_addresses`, `proposal_permalink` ) VALUES ( '$subject', '$current_dt' , '$current_tm', '$clicks', '$users_email', '$post_permalink' )";
				$wpdb->query( $sql );
			}
		}
		
		public function wolfiz_add_proposal_post_type_logs_menu(){
			
			add_menu_page( 
				'Email Notification Settings', 
				'Email Notification Settings', 
				'manage_options', 
				'wolfiz_aelieve_post_types_email_template', 
				array( &$this, 'wolfiz_post_types_email_template_callback' )
			);
			
			add_submenu_page( 
				'post_types_email_template', // to do change post type 'edit.php?post_type=proposal'
				'Mail Logs', 
				'Mail Logs',
				'manage_options',
				'mail_logs',
				array( &$this, 'wolfiz_mail_logs_callback' )
			);
			
			add_submenu_page( 
				'', // to do change post type 'edit.php?post_type=proposal'
				'Post Types', 
				'Post Types',
				'manage_options',
				'wolfiz_aelieve_post_types',
				array( &$this, 'wolfiz_email_customization_callback' )
			);
			
			
		}
		
		public function wolfiz_email_customization_tabs_name(){
			$admin_uri = admin_url();
			
			$tabs = array(
				'Email Customization' => esc_url ( $admin_uri.'admin.php?page=wolfiz_aelieve_post_types_email_template&tab=0' ),
				'Post Types' => esc_url ( $admin_uri.'admin.php?page=wolfiz_aelieve_post_types&tab=1' )
			);
			
			return $tabs;
		}
		
		
		public function wolfiz_get_email_customization_tabs(){
			$tabs = $this->wolfiz_email_customization_tabs_name();
			$tab = 0;
			if( isset( $_GET['tab'] ) ){
				$tab = (int)$_GET['tab'];
			}
			
			$i = 0;
			?>
				<h2 class="nav-tab-wrapper">
					<?php foreach( $tabs as $k => $v ): ?>
						<?php if( $tab == $i ):?>
					
							<a href = "<?php echo $v;?>" class = "nav-tab nav-tab-active"><?php echo $k; ?></a>
							
						<?php else: ?>
							
							<a href = "<?php echo $v;?>" class = "nav-tab"><?php echo $k; ?></a>

						<?php endif;?>
					<?php $i++; endforeach; ?>
				</h2>
			<?php
		}
		
		public function wolfiz_post_types_email_template_callback(){
			require_once( 'includes/admin/views/tabs/wolfiz_email_customization_tabs.php' );
		}
		
		public function wolfiz_email_customization_callback(){
			
			require_once( 'includes/admin/views/tabs/wolfiz_select_post_types.php' );
		}
		
		public function wolfiz_save_notification_email_content(){
			
			$opts = array();
			
			$email_body = $_POST['post_type_notification_email_content'];
			
			$opts['from_email_name'] = sanitize_text_field( $_POST['wolfiz_aelieve_from_email_name'] );
			
			$opts['from_email_address'] = sanitize_email( $_POST['wolfiz_aelieve_from_email_address'] );
			
			$opts['email_body'] = wp_kses_post( stripslashes( nl2br( $email_body ) ) );
			
			update_option( 'wolfiz_email_settings', $opts );
			
			wp_redirect( esc_url( admin_url( 'admin.php?page=wolfiz_aelieve_post_types_email_template&tab=0' ) ) ); exit;
		}
		
		public function wolfiz_aelieve_wp_mail_name( $from_email_name ){
			
			$opts = get_option( 'wolfiz_email_settings' );
			
			if( !empty( $opts['from_email_name'] ) ){
				
				$from_email_name = $opts['from_email_name'];
			}
			
			return $from_email_name;
		}
		
		public function wolfiz_aelieve_wp_mail_from( $from_email_address ){
			
			
			$opts = get_option( 'wolfiz_email_settings' );
			
			if( !empty( $opts['from_email_address'] ) && is_email( $opts['from_email_address'] ) ){
				
				$from_email_address = $opts['from_email_address'];
				
			}
			
			return $from_email_address;
		}
		
		
		public function wolfiz_mail_logs_callback(){
			
			global $wpdb;
			
			$row_titles = array( 'SR NO','Proposal', 'Date Sent', 'Time Sent', 'Clicks', 'Email' );
			
			$sql = "SELECT `ID`, `proposal`, `date_sent`, `time_sent`, `clicks`, `email_addresses`, `proposal_permalink` FROM `{$wpdb->prefix}mail_logs`";
			
			$logs = $wpdb->get_results( $sql );
			
			?>
			<h2>Email Logs</h2>
			<div class = "wrap">
				<table class="widefat">
					<thead>
						<tr>
							<?php foreach( $row_titles as $row_title ):?>
								<th class="row-title"><?php echo $row_title; ?></th>
							<?php endforeach;?>
						</tr>
					</thead>
					<tbody>
						<?php foreach( $logs as $log ): ?>
							<tr>
								<td><?php echo $log->ID; ?></td>
								<td><a href = "<?php echo $log->proposal_permalink; ?>" target = "_blank"><?php echo $log->proposal; ?></a></td>
								<td><?php echo $log->date_sent; ?></td>
								<td><?php echo $log->time_sent; ?></td>
								<td><?php echo $log->clicks; ?></td>
								<td><?php echo $log->email_addresses; ?></td>
							</tr>
						<?php endforeach; ?>
					</tbody>
				</table>
			</div>
			<?php
		}
		
		public function wolfiz_format_email_content_editor( $in ) {
			
			if( isset( $_GET['page'] ) && 'wolfiz_aelieve_post_types_email_template' == $_GET['page'] ) {
				//styles for the editor to provide better visual representation.
				$in['content_css'] = plugin_dir_url( __FILE__ ) . '/admin/css/main.css';
				
			}

			return $in;
		}
		
	} // end class
	
} // end if class exists

new Wolfiz_Email_Notification_Protected_Post();
?>